use std::fmt::{Display, Formatter};

#[derive(Debug)]
pub struct Ax25Frame {
    source: String,
    dest: String,
    info: String,
}

impl Ax25Frame {
    pub fn from_raw(data: &[u8]) -> Option<Self> {
        let mut iter = data.iter();

        // address
        let mut address = vec![];
        loop {
            let mut byte = *iter.next()?;
            let finished = byte & 1;
            byte >>= 1;
            address.push(byte);

            if finished == 1 {
                break;
            }
        }

        // technically we can also have addresses of length 28
        // but I haven't bothered to implement them.
        if address.len() != 14 {
            return None;
        }

        let mut source = String::new();
        for &b in &address[0..6] {
            // omit spaces
            if b != 32 {
                source.push(b as char);
            }
        }

        let mut dest = String::new();
        for &b in &address[7..13] {
            // omit spaces
            if b != 32 {
                dest.push(b as char);
            }
        }

        let _control = *iter.next()?;
        let mut info = String::new();
        for &b in iter.as_ref() {
            info.push(b as char);
        }
        Some(Self { source, dest, info })
    }
}

impl Display for Ax25Frame {
    fn fmt(&self, f: &mut Formatter) -> Result<(), std::fmt::Error> {
        write!(f, "{}>{}:{}", self.source, self.dest, self.info)
    }
}
