use bell202_decoder::{ax25::Ax25Frame, hdlc::Hdlc, iters::*, stuffer};
use std::fs::File;

const BAUD_RATE: u32 = 1200;

fn main() {
    let args: Vec<String> = std::env::args().collect();
    if args.len() != 2 {
        eprintln!("Usage {} file.wav", args[0]);
        std::process::exit(1);
    }

    let mut inp_file = File::open(&args[1]).unwrap();
    let (header, data) = wav::read(&mut inp_file).unwrap();
    assert_eq!(1, header.channel_count);
    let sample_rate = header.sampling_rate;
    println!("{}", sample_rate);
    let data = data
        .try_into_sixteen()
        .unwrap()
        .into_iter()
        .map(|x| x as f32 / std::i16::MAX as f32);

    let framestream = FrameIter::new(
        NrziIter::new(DecoderIter::new(
            DownSampleIter::new(data, sample_rate as f32, 9600.0),
            9600.0,
            BAUD_RATE,
        )),
        1000,
    )
    .filter_map(stuffer::unstuff_bits)
    .filter_map(Hdlc::new)
    .filter_map(|x| Ax25Frame::from_raw(&x.decoded_data()));

    for frame in framestream {
        println!("{}", frame);
    }
}

/*
// expects ~9600 samples/s
// https://sites.google.com/site/wayneholder/attiny-4-5-9-10-assembly-ide-and-programmer/bell-202-1200-baud-demodulator-in-an-attiny10
fn decode(samples: &[i8]) -> i16 {
    const COEFFLOI: [i16; 8] = [64, 45, 0, -45, -64, -45, 0, 45];
    const COEFFLOQ: [i16; 8] = [0, 45, 64, 45, 0, -45, -64, -45];
    const COEFFHII: [i16; 8] = [64, 8, -62, -24, 55, 39, -45, -51];
    const COEFFHIQ: [i16; 8] = [0, 63, 17, -59, -32, 51, 45, -39];

    let mut outloi = 0;
    let mut outloq = 0;
    let mut outhii = 0;
    let mut outhiq = 0;

    for i in 0..8 {
        let sample = samples[i] as i16;
        outloi += sample * COEFFLOI[i];
        outloq += sample * COEFFLOQ[i];
        outhii += sample * COEFFHII[i];
        outhiq += sample * COEFFHIQ[i];
    }

    (outhii >> 8) * (outhii >> 8) + (outhiq >> 8) * (outhiq >> 8)
        - (outloi >> 8) * (outloi >> 8)
        - (outloq >> 8) * (outloq >> 8)
}
*/
