use bell202_decoder::nrzi;
use bitvec::prelude::*;
use std::f32::consts;
use std::fs::File;
use wav::{BitDepth, Header, WAV_FORMAT_PCM};

const BAUD_RATE: u32 = 1200;
const LOW_FREQ: f32 = 1200.0;
const HIGH_FREQ: f32 = 2200.0;

fn main() {
    let args: Vec<String> = std::env::args().collect();
    if args.len() != 2 {
        eprintln!("Usage {} file.wav", args[0]);
        std::process::exit(1);
    }

    let sample_rate = 44100;
    let mut data = vec![0x7e; 11];
    // the quick brown fox jumped over the lazy dog
    // https://notblackmagic.com/bitsnpieces/ax.25/
    data.extend([
        0x7E, 0x39, 0x29, 0x76, 0x05, 0x02, 0x02, 0x07, 0x39, 0x76, 0x19, 0x51, 0x59, 0x02, 0x86,
        0xC0, 0x0F, 0x2A, 0x16, 0xA6, 0x04, 0x8E, 0xAE, 0x96, 0xC6, 0xD6, 0x04, 0x46, 0x4E, 0xF6,
        0xEE, 0x76, 0x04, 0x66, 0xF6, 0x1E, 0x04, 0x56, 0xAE, 0xB6, 0x0E, 0xCE, 0x04, 0xF6, 0x6E,
        0xA6, 0x4E, 0x04, 0x2E, 0x16, 0xA6, 0x04, 0x36, 0x86, 0x5E, 0x9E, 0x04, 0x26, 0xF6, 0x26,
        0xAC, 0xBA, 0x7E,
    ]);
    data.extend(vec![0x7e; 10]);

    let audio = encode(sample_rate, &nrzi(&BitVec::from_vec(data)));

    let header = Header::new(WAV_FORMAT_PCM, 1, sample_rate, 16);
    let mut out_file = File::create(&args[0]).unwrap();
    wav::write(header, &BitDepth::Sixteen(audio), &mut out_file).unwrap();
}

fn encode(sample_rate: u32, bits: &BitVec<u8, Msb0>) -> Vec<i16> {
    let samples_per_bit = sample_rate as f32 / BAUD_RATE as f32;
    let mut out = vec![];
    let mut phase = 0.0;
    let mut i = 0.0;
    for bit in bits {
        let end = i + samples_per_bit;
        while i < end {
            let freq = if *bit { HIGH_FREQ } else { LOW_FREQ };
            phase += freq * 2.0 * consts::PI / sample_rate as f32;
            // amplitude is arbitrarily set to 0.25
            out.push((phase.sin() * 0.25 * i16::MAX as f32) as i16);
            i += 1.0;
        }
    }

    out
}
